##########################################################################################
# Purpose: Split the dataset into a 60% training sample, a 20% in-time validation sample, and a 
# 20% out-of-time testing sample. 
##########################################################################################

# 1.5.1 - load the packages and read in data
library(data.table)
library(dplyr)
library(ggplot2)
library(plotly)
library(lubridate)

# Initialize file paths for working directories
data_stored = "C:\\ILTCI_Workshop\\Data"
data_output = "C:\\ILTCI_Workshop\\Output"

# Set working directory to where the CSV data is stored
setwd(data_stored)

# Import csv data
raw_data <- fread("ILTCI PM workshop CTR data.csv", sep=",", header=TRUE) 

# gc is garbage collector that clean random scraps from the data
# this is important with memory intensive programs
gc() 

# Set output directory to where the output will be stored
setwd(data_output)

# 1.5.2 - Add a current date variable so we can create an out-of-time holdout
# Assume that claims were incurred at the middle of year, and are bounded between 2000 and 2011 
ctr_data <- raw_data %>%
  mutate(incurred_date = ymd(paste(Incurred_Year, "-07-01", sep="")),
         study_start = incurred_date %m+% months(start_duration -1),
         study_start_date = if_else(study_start < ymd("2000-01-01"),
                                ymd("2000-01-01"),
                                if_else(study_start > ymd("2011-12-31"),
                                        ymd("2011-01-01"),
                                        study_start
                                        )
                                ),
         current_date_raw = study_start_date %m+% months(ClaimDuration - start_duration),
         current_date = if_else(current_date_raw > ymd("2011-12-31"),
                                ymd("2011-12-31"),
                                current_date_raw 
                                )                          
         ) %>% 
  select(-study_start, - current_date_raw) # remove extra variables

# Let's check out the hazard rates by current year
ctr_data %>%
  mutate(year = year(current_date))%>%
  group_by(year) %>%
  summarise(prior_rate = sum(Terminations)/sum(Exposure),
            terms = sum(Terminations)
            )

# It looks like 2011 is higher than usual
# Let's see what happens when we drop exposures greater than 2011-06-30
ctr_data %>%
  filter( current_date < ymd("2011-07-01")) %>%
  mutate(year = year(current_date))%>%
  group_by(year) %>%
  summarise(prior_rate = sum(Terminations)/sum(Exposure),
            terms = sum(Terminations)
            )

# Those rates are a bit more reasonable for workshop purposes 
# Let's drop claims greater than 2011-06-30
ctr_data <- ctr_data %>%
  filter( current_date < ymd("2011-07-01"))

# 1.5.3 - Now we can split the data
# Find the date the corresponds to the 80th percentile for the out-of-time holdout
(test_cut_date <- quantile(ctr_data$current_date, .80, type = 1))

# Grab all the distinct claim IDs in the data and tag on random numbers for sampling
# Do this at the claim level, as We do not want to split a claim across the training and validation data
distinct_claim_ids <- ctr_data %>% 
                            distinct(fake_claim_id)

# Use a seed to make results reproducible
set.seed(123)
rand_claim_ids <- data.table(distinct_claim_ids, 
                             RandNum = runif(nrow(distinct_claim_ids), 0, 1)
                             )

# Join random number onto data
ctr_data <- left_join(ctr_data, 
                      rand_claim_ids, 
                      by="fake_claim_id"
                      ) 

# Tag on sample flags
ctr_data <- ctr_data %>% 
  mutate(Sample = ifelse(current_date > test_cut_date,
                         "testing", # 20%
                         ifelse(RandNum > 3/4, 
                                "validation", # 60% = 80% * (3/4)
                                "training" # 20%
                                )
                         )
         ) %>%
  select(-RandNum)

# 1.5.4 - Now let's check out the distribution of our samples by year using barplot()
?barplot() # Use ? before function to pull up help window

# added in color field to the data
# check out list of colors here Www.stat.columbia.edu/~tzheng/files/Rcolor.pdf
ctr_data_color <- ctr_data %>%
  mutate(SampleColor = ifelse(Sample == "validation",
                              "orange", 
                              ifelse(Sample == "training",
                                     "blue4", 
                                     "chartreuse4"
                                     )
                              )
         )



# Store year and color field in a table
date_plottable <- table(ctr_data_color$SampleColor, 
                        year(ctr_data_color$current_date)
                        )

# Now let's plot this data in the Plots window
barplot(date_plottable,
        col = rownames(date_plottable),
        las = 2,
        main = "Exposure over time",
        xlab="Year",
        legend.text=c("Training", "Testing", "Validation"),
        args.legend = list(x = "topleft")
        )

# Do a quick summary of the data
ctr_data %>% 
  group_by(Sample) %>% 
  summarise(Expo_Dist = sum(Exposure)/sum(ctr_data$Exposure),
            Exposures = sum(Exposure), 
            Terminations = sum(Terminations),
            Hazard_rate = sum(Terminations)/sum(Exposure)
            )                                                         

# Plot actual termination rates
raw_hazard <- ctr_data %>%
  group_by(ClaimDuration,
           Sample) %>%
  summarise(raw_hazard_rate = sum(Terminations)/sum(Exposure))

ggplot() + 
  geom_line(data = raw_hazard, 
            aes(x =ClaimDuration , 
                y = raw_hazard_rate, 
                colour = factor(c(Sample))
                )
            )+ 
  ggtitle("Actual termination hazard rates")

# That plot is difficult to analyze because of the scale
# Let's Wrap the graph in ggplotly() to make it interactive in the Viewer window that can zoom in
ggplotly(ggplot() +
           geom_line(data = raw_hazard, 
                     aes(x =ClaimDuration , 
                         y = raw_hazard_rate, 
                         colour = factor(c(Sample))
                         )
                     )+
           ggtitle("Actual termination hazard rates")
          )

# Terminations rates fluctuate after 100 and even more so after 150
# Investigate and understand reason for spikes
# For purposes of workshop, we'll chalk up to limited exposure in the tail
# Care will be needed when analyzing the model in the tail and 
#   judgement is needed in how to use/extrapolate where there is limited data

# Save data to out working directory
save(ctr_data, file="ctr_data.RData")













