##########################################################################################
# Purpose: To be able to run a Cox proportional hazard model, and to be able to
#             review and interpret the basic results
##########################################################################################

# 2.2.1 - load the packages
library(survival)
library(broom)

# 2.2.2 - Initialize file paths for working directories 
data_stored = "C:\\ILTCI_Workshop\\Data"
data_output = "C:\\ILTCI_Workshop\\Output"

# 2.2.3 - Set working directory to where the data were output
setwd(data_output)

# 2.2.4 - import ctr_data_trn (training) table - if not available, run the KM-NA.R program
load("ctr_data_trn.RData")

# 2.2.5 - recode the data into ctr_data_ph so it is in a cleaner format for the regression
ctr_data_ph <- ctr_data_trn %>%
                          select(GroupIndicator,
                                 Gender,
                                 IncurredAgeBucket,
                                 Incurred_Year,
                                 ClaimType,
                                 Cov_Type_Bucket,
                                 Exposure,
                                 Terminations,
                                 ClaimDuration,
                                 ClaimDuration.n,
                                 Diagnosis_Category,
                                 TQ_Status,
                                 Infl_Rider_Bucket,
                                 Max_Ben_Bucket,
                                 beg_dur_ph,
                                 end_dur_ph,
                                 Region,
                                 Sample
                                  ) %>%
                          mutate(ClmDurBucket = case_when(.$ClaimDuration <= 3 ~ "Mos1-3",
                                                          .$ClaimDuration > 3 & .$ClaimDuration <= 12 ~ "Mo4-12",
                                                          .$ClaimDuration > 12 & .$ClaimDuration <= 72 ~ "Yrs2-6",
                                                          .$ClaimDuration > 72 ~ "Yrs7+ "
                                                          ),
                                 Diagnosis2   = if_else(str_detect(Diagnosis_Category, "Alzheimer") | Diagnosis_Category == "Mental",
                                                         "Mental",
                                                         "Non-Mental"
                                                          ),
                                 BP2          = if_else(Max_Ben_Bucket == "Unlimited",
                                                         "Lifetime",
                                                         "Non-Life"
                                                          )
                                  ) %>%
                          filter(ClaimType != "Unk"
                                  ) %>%
                          mutate(LnExposure = log(Exposure)
                                 )

# 2.2.6 - recode factor variables so that the base level characteristics can be more easily defined
ctr_data_ph$Gender.f <- as.factor(ctr_data_ph$Gender)
ctr_data_ph$ClaimType.f <- as.factor(ctr_data_ph$ClaimType)
ctr_data_ph$Diagnosis2.f <- as.factor(ctr_data_ph$Diagnosis2)
ctr_data_ph$Diagnosis_Category.f <- as.factor(ctr_data_ph$Diagnosis_Category)
ctr_data_ph$BP2.f <- as.factor(ctr_data_ph$BP2)
ctr_data_ph$IncurredAgeBucket.f <- as.factor(ctr_data_ph$IncurredAgeBucket)
ctr_data_ph$ClmDurBucket.f <- as.factor(ctr_data_ph$ClmDurBucket)
ctr_data_ph$Incurred_Year.f <- as.factor(ctr_data_ph$Incurred_Year)
ctr_data_ph$Region.f <- as.factor(ctr_data_ph$Region)
ctr_data_ph$TQ_Status.f <- as.factor(ctr_data_ph$TQ_Status)
ctr_data_ph$Infl_Rider_Bucket.f <- as.factor(ctr_data_ph$Infl_Rider_Bucket)
ctr_data_ph$Max_Ben_Bucket.f <- as.factor(ctr_data_ph$Max_Ben_Bucket)

# 2.2.7 - use ph data as the prefix
attach(ctr_data_ph)

# 2.2.8 - run the Cox ph regression and explicitly state the base level characteristics
results_ph <- coxph(formula = Surv(time = beg_dur_ph,
                                   time2 = end_dur_ph,
                                   event = Terminations,
                                   type = "counting"
                                  )
                                  ~ relevel(Gender.f, "Female") +
                                    relevel(IncurredAgeBucket.f,"80 to 84") +
                                    relevel(BP2.f,"Non-Life") +
                                    relevel(ClaimType.f,"HHC") +
                                    relevel(Diagnosis2.f,"Non-Mental"
                                    ),
                                  data=ctr_data_ph,
                                  ties = "efron"
                                  )

# 2.2.9 - detach the dataset
detach(ctr_data_ph)

# 2.2.10 - view the results - does everything look reasonable? 
results_ph

# 2.2.11 - store coefficients in data-frame with tidy() from the broom package, which
#           cleans up the results a little, and view the results
coef_ph <- tidy(results_ph) %>% 
                mutate(coef_exp = exp(estimate),
                       chisq_stat = statistic ** 2)
View(coef_ph)

# 2.2.12 - plot the baseline survival function - does it look as expected? remember you looked
#             at this raw survival curve in the K-M segment earlier
plot(survfit(results_ph,
             newdata = data.frame(Gender.f = "Female",
                                  IncurredAgeBucket.f = "80 to 84",
                                  BP2.f = "Non-Life",
                                  ClaimType.f = "HHC",
                                  Diagnosis2.f = "Non-Mental"
                                  )
              ),
              ylim=c(0,1),
              xlab="Months",
              ylab="Proportion not Terminated",
              main = "Base Level Survival Curve",
              col=c("black", "red", "blue")
      )
legend('topright', c("Base Level", "Lower 95%CI", "Upper 95%CI"), col=c("black", "red", "blue"), lty=1:3, cex=0.8)

# 2.2.13 - export the formatted data for future use
save(ctr_data_ph, file="ctr_data_ph.RData")

# 2.2.14 - export the Cox ph results for later use
save(results_ph, file="results_ph.RData")

