##########################################################################################
# Purpose: To be able to run a piece-wise Cox proportional hazard model, to be able to
#             review and interpret the basic results, and to show how it is equivalent
#             to a Poisson GLM
##########################################################################################

# 2.3.1 - load the packages
library(eha)

# 2.3.2 - Initialize file paths for working directories 
data_stored = "C:\\ILTCI_Workshop\\Data"
data_output = "C:\\ILTCI_Workshop\\Output"

# 2.3.3 - Set working directory to where the data were output
setwd(data_output)

# 2.3.4 - import ctr_data_ph table - if not available, run the CoxPH.R program
load("ctr_data_ph.RData")

# 2.3.5 - use ph data as the prefix
attach(ctr_data_ph)

# 2.3.6 - run the Cox piece-wise regression, it takes about a minute to run
#           note the cutpoints
results_pwe <- phreg(formula = Surv(time = beg_dur_ph,
                                   time2 = end_dur_ph,
                                   event = Terminations,
                                   type = "counting"
                                  )
                                  ~ relevel(Gender.f, "Female") +
                                    relevel(IncurredAgeBucket.f,"80 to 84") +
                                    relevel(BP2.f,"Non-Life") +
                                    relevel(ClaimType.f,"HHC") +
                                    relevel(Diagnosis2.f,"Non-Mental"
                                    ),
                                  data=ctr_data_ph,
                                  dist="pch",
                                  cut = c(3, 12, 72) # Cutpoints to approximate survival curve
                                  )

# 2.3.7 - detach the dataset
detach(ctr_data_ph)

# 2.3.8 - view the results - do they look reasonable?
results_pwe

plot(results_pwe)
results_pwe$hazards

# 2.3.9 - store coefficients in data-frame with tidy() from the broom package
#           this cleans up the formatting a little better
coef_pwe <- tidy(results_pwe$coefficients)
View(coef_pwe)

# 2.3.10 - compute the Poisson glm with reduced factors and review results
#            we're jumping ahead a tiny bit on the variable selection so that
#            we can dig into the Poisson GLM after this segment
results_ps <- glm(Terminations ~ 
                    relevel(Gender.f, "Female") 
                  + relevel(ClaimType.f,"HHC")
                  + relevel(Diagnosis2.f,"Non-Mental")
                  + relevel(BP2.f,"Non-Life")
                  + relevel(IncurredAgeBucket.f,"80 to 84")
                  + relevel(ClmDurBucket.f,"Mos1-3"),
                  data = ctr_data_ph, 
                  family = poisson, 
                  offset = LnExposure
                  )

summary(results_ps)

# 2.3.11 - to compare to phreg results, we need to exponentiate the Poisson parameters 
coef_ps_clmdur <- exp(results_ps$coefficients[1])
coef_ps_clmdur[2] <- exp(results_ps$coefficients[1]+results_ps$coefficients[14])
coef_ps_clmdur[3] <- exp(results_ps$coefficients[1]+results_ps$coefficients[15])
coef_ps_clmdur[4] <- exp(results_ps$coefficients[1]+results_ps$coefficients[16])

# 2.3.12 - show claim duration equivalence - are they the same?
results_pwe$hazards
coef_ps_clmdur

# 2.3.13 - show coefficient equivalence - are they the same?
coef_pwe
summary(results_ps)

# 2.2.14 - export the Cox pwe results for later use
save(results_pwe, file="results_pwe.RData")

