######################################################################################################
# Purpose: To be able to run a Poisson GLM and explore the data a little more, to be able
#               to review and interpret the basic results, and to understand the model's shortcomings
######################################################################################################

# 2.4.1 - load the packages
library(dplyr)
library(stringr)
library(broom)

# 2.4.2 - Initialize file paths for working directories 
data_stored = "C:\\ILTCI_Workshop\\Data"
data_output = "C:\\ILTCI_Workshop\\Output"

# 2.4.3 - Set working directory to where the data were output
setwd(data_output)

# 2.4.4 - import table - if not available, run the "Separate data into train val test.R" program and convert
#             to tbl_df format for easier viewing
load("ctr_data.RData")
ctr_data <- tbl_df(ctr_data)

# 2.4.5 - view the data structure
str(ctr_data)

# 2.4.6 - select the important fields, create a couple of new fields, and filter the data
#             first repeat the step you did in the pre-work to create the training data in program 0200
ctr_data$ClaimDuration.n <- as.numeric(ctr_data$ClaimDuration)
ctr_data_trn <- ctr_data %>%
                        mutate(beg_dur_ph = case_when(.$start_duration == .$end_duration ~
                                                        .$ClaimDuration.n - 1,
                                                      .$ClaimDuration == .$start_duration ~
                                                        .$ClaimDuration.n - .$Exposure,
                                                      .$ClaimDuration == .$end_duration ~
                                                        .$ClaimDuration.n - 1,
                                                      (.$ClaimDuration > .$start_duration) & (.$ClaimDuration < .$end_duration) ~
                                                        .$ClaimDuration.n - 1
                                                      ),
                                end_dur_ph = beg_dur_ph + .$Exposure
                               ) %>%
                        filter(GroupIndicator == "Ind",
                               Cov_Type_Bucket == "Comprehensive",
                               Sample == "training"
                              )

# 2.4.7 - then repeat the step that created the data for the proportional hazard regression in program 0220
ctr_data_ph <- ctr_data_trn %>%
                          select(GroupIndicator,
                                 Gender,
                                 IncurredAgeBucket,
                                 Incurred_Year,
                                 ClaimType,
                                 Cov_Type_Bucket,
                                 Exposure,
                                 Terminations,
                                 ClaimDuration,
                                 ClaimDuration.n,
                                 Diagnosis_Category,
                                 TQ_Status,
                                 Infl_Rider_Bucket,
                                 Max_Ben_Bucket,
                                 beg_dur_ph,
                                 end_dur_ph,
                                 Region,
                                 Sample
                                  ) %>%
                          mutate(ClmDurBucket = case_when(.$ClaimDuration <= 3 ~ "Mos1-3",
                                                          .$ClaimDuration > 3 & .$ClaimDuration <= 12 ~ "Mo4-12",
                                                          .$ClaimDuration > 12 & .$ClaimDuration <= 72 ~ "Yrs2-6",
                                                          .$ClaimDuration > 72 ~ "Yrs7+ "
                                                          ),
                                 Diagnosis2   = if_else(str_detect(Diagnosis_Category, "Alzheimer") | Diagnosis_Category == "Mental",
                                                         "Mental",
                                                         "Non-Mental"
                                                          ),
                                 BP2          = if_else(Max_Ben_Bucket == "Unlimited",
                                                         "Lifetime",
                                                         "Non-Life"
                                                          )
                                  ) %>%
                          filter(ClaimType != "Unk"
                                  ) %>%
                          mutate(LnExposure = log(Exposure)
                                 )

# 2.4.8 - repeat the steps to recode the factor variables so that the base level characteristics
#             can be more easily defined
ctr_data_ph$Gender.f <- as.factor(ctr_data_ph$Gender)
ctr_data_ph$ClaimType.f <- as.factor(ctr_data_ph$ClaimType)
ctr_data_ph$Diagnosis2.f <- as.factor(ctr_data_ph$Diagnosis2)
ctr_data_ph$Diagnosis_Category.f <- as.factor(ctr_data_ph$Diagnosis_Category)
ctr_data_ph$BP2.f <- as.factor(ctr_data_ph$BP2)
ctr_data_ph$IncurredAgeBucket.f <- as.factor(ctr_data_ph$IncurredAgeBucket)
ctr_data_ph$ClmDurBucket.f <- as.factor(ctr_data_ph$ClmDurBucket)
ctr_data_ph$Incurred_Year.f <- as.factor(ctr_data_ph$Incurred_Year)
ctr_data_ph$Region.f <- as.factor(ctr_data_ph$Region)
ctr_data_ph$TQ_Status.f <- as.factor(ctr_data_ph$TQ_Status)
ctr_data_ph$Infl_Rider_Bucket.f <- as.factor(ctr_data_ph$Infl_Rider_Bucket)
ctr_data_ph$Max_Ben_Bucket.f <- as.factor(ctr_data_ph$Max_Ben_Bucket)


# 2.4.9 - aggregate the ph data into ctr_data_ps; this wil speed up the regression significantly,
#           since there are fewer records in the consolidated data
ctr_data_ps <- ctr_data_ph %>%
                          group_by(Cov_Type_Bucket,
                                   Gender.f, 
                                   IncurredAgeBucket.f, 
                                   Incurred_Year, 
                                   Incurred_Year.f,
                                   ClaimType.f,
                                   ClmDurBucket.f,
                                   Diagnosis2.f,
                                   BP2.f,
                                   Sample,
                                   Diagnosis_Category.f, 
                                   TQ_Status.f,          
                                   Infl_Rider_Bucket.f,  
                                   Max_Ben_Bucket.f,     
                                   Region.f        
                                   ) %>%
                          summarise(SumExposure = sum(Exposure), 
                                    SumTerminations = sum(Terminations)
                                    ) %>% data.frame

# 2.4.10 - re-calc the lnexposure column, and a new incurred year variable
#           this needs to be done because these drivers are not additive
ctr_data_ps <- mutate(ctr_data_ps, 
                    LnExposure = log(SumExposure), 
                    LnIncYr = log(Incurred_Year - 1990)
                    )

# 2.4.11 - attach the table name
attach(ctr_data_ps)

# 2.4.12 - run a basic poisson glm with three driver variables of gender, situs, and claim duration
#             view the results
results_ps <- glm(SumTerminations ~ 
                    relevel(Gender.f,"Female")
                  + relevel(ClaimType.f,"HHC")
                  + relevel(ClmDurBucket.f,"Mos1-3"),
                  data = ctr_data_ps, 
                  family = poisson, 
                  offset = LnExposure
                  )

coef_ps <- tidy(results_ps) %>% 
                        mutate(coef_exp = exp(estimate),
                               chisq_stat = statistic ** 2)

summary(results_ps)

# 2.4.13 - breakout exercise - try out some of the variable listed above (rows 88 to 99)
#             to see if they improve the explanatory power of the GLM
#             would you keep those variables and why?


# 2.4.14 - compute a poisson glm with an expanded list of drivers and review results
#           which variables would you keep, and why?
results_ps <- glm(SumTerminations ~ 
                    relevel(Gender.f,"Female")
                  + relevel(ClaimType.f,"HHC")
                  + relevel(IncurredAgeBucket.f,"80 to 84")
                  + relevel(ClmDurBucket.f,"Mos1-3")
                  + relevel(Region.f, "Unknown")
                  + relevel(Diagnosis_Category.f, "Unknown")
                  + relevel(TQ_Status.f, "U")
                  + relevel(Infl_Rider_Bucket.f, "Unknown")
                  + relevel(Max_Ben_Bucket.f, "Unknown"),
                  data = ctr_data_ps, 
                  family = poisson, 
                  offset = LnExposure
                  )

coef_ps <- tidy(results_ps) %>% 
                        mutate(coef_exp = exp(estimate),
                               chisq_stat = statistic ** 2)

summary(results_ps)

# 2.4.15 - view a Type III analysis of the results
drop1(results_ps,~.,test="Chisq")

# 2.4.16 - re-aggregate the data with reduced factors into ctr_data_ps
#           we removed some variables, so we can even further consolidate
#           the data for quicker downstream processing
ctr_data_ps <- ctr_data_ps %>%
                        select(Cov_Type_Bucket,
                               Gender.f, 
                               IncurredAgeBucket.f, 
                               Incurred_Year,
                               Incurred_Year.f,
                               ClaimType.f, 
                               ClmDurBucket.f, 
                               Diagnosis2.f, 
                               BP2.f,
                               Sample,
                               SumExposure,
                               SumTerminations
                              ) %>%
                        group_by(Cov_Type_Bucket,
                                 Gender.f, 
                                 IncurredAgeBucket.f, 
                                 Incurred_Year,
                                 Incurred_Year.f,
                                 ClaimType.f, 
                                 ClmDurBucket.f,
                                 Diagnosis2.f, 
                                 BP2.f,
                                 Sample
                                ) %>%
                        summarise(SumExposure = sum(SumExposure), 
                                  SumTerminations = sum(SumTerminations)
                                  ) %>% data.frame

# 2.4.17 - re-calc the lnexposure column, and a new incurred year variable (non-additive)
ctr_data_ps <- mutate(ctr_data_ps, 
                      LnExposure = log(SumExposure), 
                      LnIncYr = log(Incurred_Year - 1990)
                      ) %>% data.frame()

# 2.4.18 - compute the poisson glm with incurred year and review results
#           what should we do with the incurred year variable?
results_ps <- glm(SumTerminations ~ 
                    relevel(Gender.f, "Female") 
                  + relevel(ClaimType.f,"HHC")
                  + relevel(Diagnosis2.f,"Non-Mental")
                  + relevel(BP2.f,"Non-Life")
                  + relevel(IncurredAgeBucket.f,"80 to 84")
                  + relevel(ClmDurBucket.f,"Mos1-3")
                  + relevel(Incurred_Year.f,"2009"),
                  data = ctr_data_ps, 
                  family = poisson, 
                  offset = LnExposure
                  )

summary(results_ps)

# 2.4.19 - compute the poisson glm with fitted incurred year and review results
#           how did we do on the incurred year variable?
results_ps <- glm(SumTerminations ~ 
                    relevel(Gender.f, "Female") 
                  + relevel(ClaimType.f,"HHC")
                  + relevel(Diagnosis2.f,"Non-Mental")
                  + relevel(BP2.f,"Non-Life")
                  + relevel(IncurredAgeBucket.f,"80 to 84")
                  + relevel(ClmDurBucket.f,"Mos1-3")
                  + LnIncYr,
                  data = ctr_data_ps, 
                  family = poisson, 
                  offset = LnExposure
                  )

summary(results_ps)

# 2.4.20 - review the parameter correlation matrix; should we think about adding
#           some cross-terms as a result of this review?
corresult <- summary(results_ps,
                     correlation = TRUE
                     )

corrmat <- corresult$correlation
View(corrmat)

# 2.4.21 - run the final model with interactions and review results
#           are the new interaction terms worth keeping?
results_ps <- glm(SumTerminations ~ 
                    relevel(Gender.f, "Female") 
                  + relevel(ClaimType.f,"HHC")
                  + relevel(Diagnosis2.f,"Non-Mental")
                  + relevel(BP2.f,"Non-Life")
                  + LnIncYr * relevel(IncurredAgeBucket.f,"80 to 84")
                  + LnIncYr * relevel(ClmDurBucket.f,"Mos1-3"),
                  data = ctr_data_ps, 
                  family = poisson, 
                  offset = LnExposure
                  )

summary(results_ps)

# 2.4.22 - display the dispersion parameter - this is always "1" for a regular Poisson
#           we can potentially relax this parameter as a better choice of distribution
summary.glm(results_ps)$dispersion

# 2.4.23 - Type I analysis, which tells you the sequential explanatory impact of
#         adding the driver variables - order matters!
anova(results_ps)

# 2.4.24 - Type III analysis, which tells you the explanatory impact of removing each variable (one at a time)
#         while keeping all the others - order does not matter
drop1(results_ps,~.,test="Chisq")

# 2.4.25 - export the poisson glm results for later use, and save the data
save(results_ps, file="results_ps.RData")
save(ctr_data_ps, file="ctr_data_ps.RData")

# 2.4.26 - try to over-fit the data using the gender, situs, and claim duration drivers
results_ps <- glm(SumTerminations ~ 
                    relevel(Gender.f, "Female") 
                  * relevel(ClaimType.f,"HHC")
                  * relevel(ClmDurBucket.f,"Mos1-3"),
                  data = ctr_data_ps, 
                  family = poisson, 
                  offset = LnExposure
                  )

summary(results_ps)

# 2.4.27 - let's see how the A to E results look - too perfect a fit means that
#           the model will likely not test well out-of-sample
ctr_data_ps$fitted <- results_ps$fitted.values

gender_situs_dur <- ctr_data_ps %>%
                            group_by(Gender.f, 
                                     ClaimType.f,
                                     ClmDurBucket.f
                                     ) %>%
                            summarise(actual_hz_rate = sum(SumTerminations)/sum(SumExposure),
                                      exp_hz_rate = sum(fitted)/sum(SumExposure),
                                      terms = sum(SumTerminations),
                                      fitted = sum(fitted),
                                      AtoE = sum(SumTerminations)/sum(fitted)) %>% data.frame()

View(gender_situs_dur)

