######################################################################################################
# Purpose: To be able to run a Poisson GLM and explore the data a little more, to be able
#               to review and interpret the basic results, and to understand the model's shortcomings
######################################################################################################

# 2.5.1 - load the packages
library(dplyr)
library(stringr)
library(broom)

# 2.5.2 - Initialize file paths for working directories 
data_stored = "C:\\ILTCI_Workshop\\Data"
data_output = "C:\\ILTCI_Workshop\\Output"

# 2.5.3 - Set working directory to where the data were output
setwd(data_output)

# 2.5.4 - import table - if not available, run the "PoissonGLM Explore.R" program and convert
#             to tbl_df format for easier viewing
load("ctr_data_ph.RData")
ctr_data_ph <- tbl_df(ctr_data_ph)

# 2.5.5 - view the data structure
str(ctr_data_ph)

# 2.5.6 - Load in starting E that varies by duration and gender and tag it onto the data
# Note that these are hazard rates. If they were the probability of termination qx, we would 
# need to convert them to hazard rates (hazard_rate = -log(1-qx)) 
setwd(data_stored)
load("smoothed_assumptions_20180301.RData")

# 2.5.7 - view the data structure
str(smoothed_assumptions)

# 2.5.8 - Join on our smoothed assumption to the data
# Note that the exp_terms is the smoothed_assumption multiplied by the exposures
# This gives us the expected claim terminations
ctr_data_ps <- ctr_data_ph %>% 
                        left_join(smoothed_assumptions,
                                  by = c("ClaimDuration", "Gender")
                                  ) %>% 
                        mutate(exp_terms = smoothed*Exposure)

# 2.5.9 - Review the A/E results by gender and claim duration groupings
gender_duration <- ctr_data_ps %>%
                          group_by(Gender, 
                                   ClmDurBucket
                                   ) %>%
                          summarise(actual_hz_rate = sum(Terminations)/sum(Exposure),
                                    exp_hz_rate = sum(exp_terms)/sum(Exposure),
                                    terms = sum(Terminations),
                                    expected = sum(exp_terms),
                                    AtoE = sum(Terminations)/sum(exp_terms)) %>% data.frame()

View(gender_duration)

# 2.5.10 - Looks like it is a bit high - let's take a look by situs
situs <- ctr_data_ps %>%
                          group_by(ClaimType
                                   ) %>%
                          summarise(actual_hz_rate = sum(Terminations)/sum(Exposure),
                                    exp_hz_rate = sum(exp_terms)/sum(Exposure),
                                    terms = sum(Terminations),
                                    expected = sum(exp_terms),
                                    AtoE = sum(Terminations)/sum(exp_terms)) %>% data.frame()

View(situs)

# 2.5.11 - Let's run a glm with offset to see the same phenomenon 
# Calculate log exposure of the expected terminations, which are needed for the glm
ctr_data_ps <- ctr_data_ps %>% 
  mutate(Ln_exp_terms = log(exp_terms))


# 2.5.12 - Run a simple GLM with an expected basis offset (Ln_exp_terms) and view the results
results_off <- glm(Terminations ~ 
                    relevel(Gender.f,"Female")
                  + relevel(ClaimType.f,"HHC")
                  + relevel(ClmDurBucket.f,"Mos1-3"),
                  data = ctr_data_ps, 
                  family = poisson, 
                  offset = Ln_exp_terms
                  )

coef_off <- tidy(results_off) %>% 
                    mutate(coef_exp = exp(estimate),
                           chisq_stat = statistic ** 2)

summary(results_off)
View(coef_off)

# 2.5.13 - Let's see what happens if we make "NH" the base level for situs instead of HHC
results_off <- glm(Terminations ~ 
                    relevel(Gender.f,"Female")
                  + relevel(ClaimType.f,"NH")
                  + relevel(ClmDurBucket.f,"Mos1-3"),
                  data = ctr_data_ps, 
                  family = poisson, 
                  offset = Ln_exp_terms
                  )

coef_off <- tidy(results_off) %>% 
                    mutate(coef_exp = exp(estimate),
                           chisq_stat = statistic ** 2)

summary(results_off)
View(coef_off)

# 2.5.14 - Let's look at the A to E results from the new model
ctr_data_ps$fitted <- results_off$fitted.values

gender_duration <- ctr_data_ps %>%
                            group_by(Gender,
                                     ClmDurBucket
                                     ) %>%
                            summarise(actual_hz_rate = sum(Terminations)/sum(Exposure),
                                      exp_hz_rate = sum(fitted)/sum(Exposure),
                                      terms = sum(Terminations),
                                      fitted = sum(fitted),
                                      AtoE = sum(Terminations)/sum(fitted)) %>% data.frame()
  
View(gender_duration)

# 2.5.15 - And how about by situs and claim duration bucket - looks like there's still room for improvement
situs_duration <- ctr_data_ps %>%
                            group_by(ClaimType,
                                     ClmDurBucket
                                     ) %>%
                            summarise(actual_hz_rate = sum(Terminations)/sum(Exposure),
                                      exp_hz_rate = sum(fitted)/sum(Exposure),
                                      terms = sum(Terminations),
                                      expected = sum(fitted),
                                      AtoE = sum(Terminations)/sum(fitted)) %>% data.frame()

View(situs_duration)

