######################################################################################################
# Purpose: To be able to verify that the Cox and Poisson models have a reasonable fit, and whether
#               or not the model violates any basic a priori assumptions
######################################################################################################

# 3.0.1 - load the packages
library(AER)
library(car)
library(lmtest)
library(zoo)
library(ggplot2)

# 3.0.2 - Initialize file paths for working directories 
data_stored = "C:\\ILTCI_Workshop\\Data"
data_output = "C:\\ILTCI_Workshop\\Output"

# 3.0.3 - Set working directory to where the output was saved
setwd(data_output)

# 3.0.4 - load the prior Cox and Poisson regression results - run the prior .R files if they do not yet exist
load("results_ph.RData")
load("results_pwe.RData")
load("results_ps.RData")

# 3.0.5 - also load the data files, which are needed to run some of the diagnostics
#           some of the prior .R programs will need to be run if the data do not exist
load("ctr_data_ph.RData")
load("ctr_data_ps.RData")

# 3.0.6 - test for non-proportional hazards in the more refined Cox model results
test_ph <- cox.zph(results_ph, transform = "log")

# 3.0.7 - view the nph test results - does anything jump out at you?
test_ph

# 3.0.8 - plot the nph test results for the gender and mental effects
#           bends in the residuals indicate non-proportional hazards
plot(test_ph[1],
     xlab = "log(Time)",
     ylab = "Gender residuals",
     main = "Non-Proportional Hazard Test for Gender"
     )

plot(test_ph[12],
     xlab = "log(Time)",
     ylab = "Diagnosis residuals",
     main = "Non-Proportional Hazard Test for Diagnosis"
     )

# 3.0.9 - check for over-dispersion - this tells us whether a Poisson model is rejected
dispersiontest(results_ps, trafo = 1)

# 3.0.10 - re-run model with over-dispersed poisson, review results - what changed?
results_qp <- glm(SumTerminations ~ 
                    relevel(Gender.f, "Female") 
                  + relevel(ClaimType.f,"HHC")
                  + relevel(Diagnosis2.f,"Non-Mental")
                  + relevel(BP2.f,"Non-Life")
                  + LnIncYr*relevel(IncurredAgeBucket.f,"80 to 84")
                  + LnIncYr*relevel(ClmDurBucket.f,"Mos1-3"),
                  data = ctr_data_ps, 
                  family = quasipoisson, 
                  offset = LnExposure
                  )

summary(results_qp)
summary(results_ps)

# 3.0.11 - display the dispersion parameter - notice it is no longer "1"
summary.glm(results_qp)$dispersion

# 3.0.12 - retrieve the quasi-Poisson correlation matrix, fitted values vs. actuals, and dfb for outliers
#           these wil help us see if we can identify any other problems with the model
corresult <- summary(results_qp, correlation = TRUE)
corrmat <- corresult$correlation
Fit <- results_qp$fitted.values
resids <- results_qp$residuals
dfb <- dfbeta(results_qp)

# 3.0.13 - review fitted values - a straight line going from SW to NE is good
ctr_data_ps$fit <- results_qp$fitted.values

ggplot(ctr_data_ps,
                 aes(x=Fit,
                    y=SumTerminations
                    )
       ) + geom_point(size = 2) +
         ggtitle("Actual vs. Fit Termination Counts")

# 3.0.14 - show df beta test for outliers for the gender variable - any outliers?
dfb_gender <- dfb[,2]
plot(dfb_gender,
     xlab = "Observation",
     ylab = "Coefficient Sensitivity",
     main = "DF Beta Chart for the Gender Coefficient"
     )

# 3.0.15 - calculate A/E ratios by incurred year
AtoE_m <- ctr_data_ps %>%
                  select(Gender.f, 
                         SumExposure,
                         SumTerminations, 
                         Incurred_Year,
                         fit
                          ) %>%
                  filter(Gender.f == "Male"
                          ) %>%
                  group_by(Incurred_Year
                          ) %>%
                  summarise(SumExposure = sum(SumExposure), 
                            SumTerminations = sum(SumTerminations),
                            SumFitted = sum(fit)
                          )

AtoE_f <- ctr_data_ps %>%
                  select(Gender.f, 
                         SumExposure,
                         SumTerminations, 
                         Incurred_Year,
                         fit
                          ) %>%
                  filter(Gender.f == "Female"
                          ) %>%
                  group_by(Incurred_Year
                          ) %>%
                  summarise(SumExposure = sum(SumExposure), 
                            SumTerminations = sum(SumTerminations),
                            SumFitted = sum(fit)
                          )

# 3.0.16 - view the actual vs. fit results graphically - any problems?
plot(AtoE_m$SumTerminations,
     AtoE_m$SumFitted,
     xlab = "Actual Counts",
     ylab = "Fitted Counts",
     main = "Actual vs. Fitted Terminations for Males, Training Set")

plot(AtoE_f$SumTerminations,
     AtoE_f$SumFitted,
     xlab = "Actual Counts",
     ylab = "Fitted Counts",
     main = "Actual vs. Fitted Terminations for Females, Training Set")

# 3.0.17 - view the A/E ratios by incurred year - how do these look?
AtoE_mr <- tbl_df(AtoE_m$SumTerminations / AtoE_m$SumFitted)
AtoE_fr <- tbl_df(AtoE_f$SumTerminations / AtoE_f$SumFitted)

barplot(AtoE_mr$value,
        xlab = "Incurred Year",
        ylab = "A/E Ratio",
        main = "A/E ratio by Incurred Year for Males, Training Set"
        )
barplot(AtoE_fr$value,
        xlab = "Incurred Year",
        ylab = "A/E Ratio",
        main = "A/E ratio by Incurred Year for Females, Training Set"
        )

# 3.0.18 - review out of sample results in the validation dataset to see how the model performs
#          out of sample - load the original dataset and re-process the filters and re-coding
load("ctr_data.RData")

ctr_data_qp_val <- ctr_data %>%
                      select(GroupIndicator, 
                             Gender, 
                             IncurredAgeBucket, 
                             Incurred_Year, 
                             ClaimType, 
                             Cov_Type_Bucket, 
                             Exposure,
                             Terminations, 
                             ClaimDuration, 
                             Diagnosis_Category, 
                             Max_Ben_Bucket,
                             Sample
                             ) %>%
                      mutate(ClmDurBucket = case_when(.$ClaimDuration <= 3 ~ "Mos1-3",
                                                      .$ClaimDuration > 3 & .$ClaimDuration <= 12 ~ "Mo4-12",
                                                      .$ClaimDuration > 12 & .$ClaimDuration <= 72 ~ "Yrs2-6",
                                                      .$ClaimDuration > 72 ~ "Yrs7+ "
                                                            ),
                             Diagnosis2   = if_else(str_detect(Diagnosis_Category, "Alzheimer") | Diagnosis_Category == "Mental",
                                                   "Mental", 
                                                   "Non-Mental"
                                                    ),
                             BP2          = if_else(Max_Ben_Bucket == "Unlimited", 
                                                   "Lifetime", 
                                                   "Non-Life"
                                                    )
                            ) %>%
                      filter(GroupIndicator == "Ind", 
                             ClaimType != "Unk", 
                             Cov_Type_Bucket == "Comprehensive",
                             Sample == "validation"
                             ) %>%
                      group_by(Gender, 
                               IncurredAgeBucket, 
                               Incurred_Year, 
                               ClaimType, 
                               Cov_Type_Bucket, 
                               ClmDurBucket,
                               Diagnosis2, 
                               BP2
                               ) %>%
                      summarise(SumExposure = sum(Exposure), 
                                SumTerminations = sum(Terminations)
                               )
                    
ctr_data_qp_val <- mutate(ctr_data_qp_val, 
                      LnExposure = log(SumExposure), 
                      LnIncYr = log(Incurred_Year - 1990)
                      )

# 3.0.19 - re-format the factor variables that are needed, so we can be explicit about what is base level
ctr_data_qp_val$Gender.f <- as.factor(ctr_data_qp_val$Gender)
ctr_data_qp_val$ClaimType.f <- as.factor(ctr_data_qp_val$ClaimType)
ctr_data_qp_val$Diagnosis2.f <- as.factor(ctr_data_qp_val$Diagnosis2)
ctr_data_qp_val$BP2.f <- as.factor(ctr_data_qp_val$BP2)
ctr_data_qp_val$IncurredAgeBucket.f <- as.factor(ctr_data_qp_val$IncurredAgeBucket)
ctr_data_qp_val$ClmDurBucket.f <- as.factor(ctr_data_qp_val$ClmDurBucket)
ctr_data_qp_val$Incurred_Year.f <- as.factor(ctr_data_qp_val$Incurred_Year)
  
# 3.0.20 - transfer the predicted values into the results object - a simple command produces these
out_of_sample_qp <- exp(predict(results_qp, ctr_data_qp_val))
ctr_data_qp_val$Fit <- out_of_sample_qp

# 3.0.21 - view the out of sample fit - it looks a little strange
ggplot(ctr_data_qp_val,
       aes(x=Fit,
           y=SumTerminations
           )
       ) + geom_point(size = 2) +
           ggtitle("Actual vs. Fit Termination Counts")

# 3.0.22 - the scale might be deceptive, let's re-scale the chart
ggplot(ctr_data_qp_val,
       aes(x=Fit,
           y=SumTerminations
           )
       )  + xlim(0, 45) + ylim(0,45) + geom_point(size = 2) +
            ggtitle("Actual vs. Fit Termination Counts")

# 3.0.23 - calculate A/E ratios by incurred year for the validation dataset
AtoE_m <- ctr_data_qp_val %>%
                        select(Gender, 
                               SumExposure,
                               SumTerminations, 
                               Incurred_Year,
                               Fit
                               ) %>%
                        filter(Gender == "Male"
                               ) %>%
                        group_by(Incurred_Year
                                ) %>%
                        summarise(SumExposure = sum(SumExposure), 
                                  SumTerminations = sum(SumTerminations),
                                  SumFitted = sum(Fit)
                                  )

AtoE_f <- ctr_data_qp_val %>%
                        select(Gender, 
                               SumExposure,
                               SumTerminations, 
                               Incurred_Year,
                               Fit
                               ) %>%
                        filter(Gender == "Female"
                              ) %>%
                        group_by(Incurred_Year
                                ) %>%
                        summarise(SumExposure = sum(SumExposure), 
                                  SumTerminations = sum(SumTerminations),
                                  SumFitted = sum(Fit)
                                 )

# 3.0.24 - view the actual vs. fit results graphically - any issues?
plot(AtoE_m$SumTerminations,
     AtoE_m$SumFitted,
     xlab = "Actual Counts",
     ylab = "Fitted Counts",
     main = "Actual vs. Fitted Terminations for Males, Validation Set"
     )

plot(AtoE_f$SumTerminations,
     AtoE_f$SumFitted,
     xlab = "Actual Counts",
     ylab = "Fitted Counts",
     main = "Actual vs. Fitted Terminations for Females, Validation Set"
     )

# 3.0.25 - view the A/E ratios by incurred year - how do those look?
AtoE_fr <- tbl_df(AtoE_f$SumTerminations / AtoE_f$SumFitted)
AtoE_mr <- tbl_df(AtoE_m$SumTerminations / AtoE_m$SumFitted)

barplot(AtoE_fr$value,
        xlab = "Incurred Year",
        ylab = "A/E Ratio",
        main = "A/E ratio by Incurred Year for Females, Validation Set"
        )

barplot(AtoE_mr$value,
        xlab = "Incurred Year",
        ylab = "A/E Ratio",
        main = "A/E ratio by Incurred Year for Males, Validation Set"
        )

# 3.0.26 - remove the first few years from the male graph - that looks a little better
barplot(AtoE_mr$value[3:17],
        xlab = "Incurred Year",
        ylab = "A/E Ratio",
        main = "A/E ratio by Incurred Year for Males, Validation Set"
        )
