# Purpose: Continuing on from the 0410 program we will now
# try and vary our assumption by additional variables.  Instead 
# of starting with the existing assumption from the 
# "smoothed_assumptions_20180301.RData" file we will now use the 
# adjusted assumption from the "smoothed_assump_adj.RData"
# file as it is now more representative of our experience.
# We will explore adding in ClaimType and a calendar year variable.
#
# Prerequisite programs:
# 0150 - Separate data into train val test.R
# 0410 - Update existing assumption with glmnet.R
################################################################

# 4.2.0 - Load the packages and explore the data
library(dplyr)
library(glmnet)
library(stringr)
library(Matrix)
library(doParallel)
library(ggplot2)
library(plotly)
library(scales)
library(lubridate)

# Initialize file paths for working directories
data_stored = "C:\\ILTCI_Workshop\\Data"
data_output = "C:\\ILTCI_Workshop\\Output"

# Load in the smoothed assumption we adjusted by gender and duration
load(file = paste0(data_output, "\\", "smoothed_assump_adj.RData"))

# Load prior processed data from the "0150 - Separate data into train val test.R" program 
load(paste0(data_output, "\\", "ctr_data.RData"))

# Apply filters used earlier
ctr_data <- ctr_data %>%
  filter(GroupIndicator == "Ind", 
         ClaimType != "Unk",
         Cov_Type_Bucket == "Comprehensive"
         )

# Convert Gender back to a character format so we can join it onto the data
smoothed_assump_adj$Gender <- as.character(smoothed_assump_adj$Gender)

# Join on the smoothed adjusted assumption from the 
# "0410 - Update existing assumption with glmnet.R" program to the data.
ctr_data_bench <- ctr_data %>% 
  left_join(smoothed_assump_adj,
            by = c("ClaimDuration", "Gender")
            ) %>% 
  mutate(exp_terms_orig = smoothed*Exposure,
         exp_terms = smooth_adj*Exposure)

rm(ctr_data)
gc()

# 4.2.1.1 - Now that we have the expected terminations tagged on the data let's explore 
# the fit of the assumption. We've already explored the fit by gender so let's look at 
# some new cuts of the data so first create some additional variables.
ctr_data_bench <- ctr_data_bench %>%
  mutate(CY_Yr = year(current_date),  # calendar year variable
         LnCY_Yr = log(year(current_date) - 1999)
         )

# 4.2.1.2 - View fit by ClaimType, which looks like it could use some adjusting
AtoE_train_type <- ctr_data_bench %>%
  filter(Sample == "training") %>%
  group_by(ClaimType) %>%
  summarise(Termination = comma(sum(Terminations)),
            Rate = percent(sum(Terminations)/sum(Exposure)),
            AtoE = round(sum(Terminations)/sum(exp_terms), 2))
View(AtoE_train_type)

# 4.2.1.3 - View fit by calendar year 
# It looks like calendar year has a downward sloping adjustment trends.  
# When using such a variable in the model it is important to view the trend 
# and determine how far you should let it extrapolate into the future.
# You will have to use judgment to determine if the trend should continue 
# or be held constant at as point.
AtoE_train_CY_Yr <- ctr_data_bench %>%
  filter(Sample == "training") %>%
  group_by(CY_Yr) %>%
  summarise(Termination = comma(sum(Terminations)),
            Rate = percent(sum(Terminations)/sum(Exposure)),
            AtoE = round(sum(Terminations)/sum(exp_terms), 2))
View(AtoE_train_CY_Yr)

ggplotly(ggplot() + 
           geom_point(data = AtoE_train_CY_Yr, 
                      aes(x = CY_Yr, y = AtoE))+ 
           geom_smooth(data = AtoE_train_CY_Yr, 
                       aes(x = CY_Yr, y = AtoE),
                       method='lm', se = FALSE)+
           scale_y_continuous(limits = c(0.5, 1.75))+
           ggtitle("AtoE by Calendar Year and gender"))
gc()

# 4.2.1.4 - To include a calendar year we must standardize the variable first
# in order for the penalization to be applied evenly across the coefficients.  Typically
# you should leave binary (0/1) variables and standardize the continuous variables. One 
# method of standardization is to normalize a variable by subtracting its mean
# and dividing it by its standard deviation.  This then centers the variable and 
# gives it a variance of one.  For count variables, taking the square root is usually 
# a good method as there tends to be diminishing returns as the count increases.
# Also transforming the variable to be bounded between 0 and 1 is a reasonable 
# standardization technique. In the end the goal is to standardize the variables such
# that it is plausible that the resulting coefficient can be drawn from the same underlying
# distribution. For example, small coefficients should have a large frequency, while
# large coefficients should have a lower frequency. Non-penalized glms are scale 
# invariant meaning that standardizing the variables will not change your predictions. 
# However, with the penalized glm the scale of the variables is
# important because the penalization puts an extra constraint on the sum of the
# coefficients. Therefore, if the scales of your variables are all different 
# the penalization will not be applied evenly across the variables.
LnCY_Yr_avg <- mean(ctr_data_bench[ctr_data_bench$Sample == "training",]$LnCY_Yr)
LnCY_Yr_st_dev <- var(ctr_data_bench[ctr_data_bench$Sample == "training",]$LnCY_Yr)^.5

ctr_data_bench$LnCY_Yr_st <- (ctr_data_bench$LnCY_Yr - LnCY_Yr_avg)/LnCY_Yr_st_dev

# check that we get a mean=0 and var=1
mean(ctr_data_bench[ctr_data_bench$Sample == "training", ]$LnCY_Yr_st)
var(ctr_data_bench[ctr_data_bench$Sample == "training", ]$LnCY_Yr_st)


# 4.2.1.5 - View fit by calendar year and other variables to see
# if there are trends within cohorts

# View gender and calendar year. We can see both genders have downward 
# sloping adjustments.
AtoE_train_g_CY <- ctr_data_bench %>%
  filter(Sample == "training") %>%
  group_by(Gender,
           CY_Yr) %>%
  summarise(Termination = comma(sum(Terminations)),
            Rate = percent(sum(Terminations)/sum(Exposure)),
            AtoE = round(sum(Terminations)/sum(exp_terms), 2))
View(AtoE_train_g_CY)

ggplotly(ggplot() + 
           geom_point(data = AtoE_train_g_CY, 
                      aes(x = CY_Yr, y = AtoE, col=c(Gender)))+ 
           geom_smooth(data = AtoE_train_g_CY, method='lm', se = FALSE,
                       aes(x = CY_Yr, y = AtoE, col=c(Gender)))+
           scale_y_continuous(limits = c(0.5, 1.75))+
           ggtitle("AtoE by Calendar Year and gender"))
gc()

# View ClaimType and calendar year. We can see ALF trends slightly upwards 
# while HHC and NH trend downwards with HHC having a large downward trend.
AtoE_train_type_CY <- ctr_data_bench %>%
  filter(Sample == "training") %>%
  group_by(ClaimType,
           CY_Yr) %>%
  summarise(Termination = comma(sum(Terminations)),
            Rate = percent(sum(Terminations)/sum(Exposure)),
            AtoE = round(sum(Terminations)/sum(exp_terms), 2))
View(AtoE_train_g_CY)

ggplotly(ggplot() + 
           geom_point(data = AtoE_train_type_CY, 
                      aes(x = CY_Yr, y = AtoE, col=c(ClaimType)))+ 
           geom_smooth(data = AtoE_train_type_CY, 
                       aes(x = CY_Yr, y = AtoE, col=c(ClaimType)),
                       method='lm', se = FALSE)+
           scale_y_continuous(limits = c(0.5, 1.75))+
           ggtitle("AtoE by Calendar Year and ClaimType"))
gc()


# 4.2.2.1 - Similar to the 0410 program we'll update the fit based on the training 
# data using a penalized glm. However, now we will explore developing a model
# with the following variables: ClaimDuration, Gender, ClaimType and CY_Yr.
# We will select the optimal penalty value using a 10-fold cross-validation (CV).
# To do this first we need create random fold IDs to assign observations randomly 
# to the 10 different folds.

# Use a variable as a place holder for the number of folds
nfolds <- 10

# Grab list of distinct claim IDs since we will create the fold by claim ID 
# in order to avoid splitting one claim into two or more folds
distinct_claim_ids <- ctr_data_bench %>%  
  distinct(fake_claim_id)

# Use seed to make results reproducible and sample 10 folds
set.seed(28)
folds <- data.frame(distinct_claim_ids,
                    fold = sample(nfolds, 
                                  size = nrow(distinct_claim_ids), 
                                  replace = TRUE
                                  )
                    )

# Join folds assignments back to the data
ctr_data_bench <- ctr_data_bench %>%
  left_join(folds, by="fake_claim_id")

rm(distinct_claim_ids,
   folds)

# Verify that each fold has about the same amount of data
round(table(ctr_data_bench$fold)/(nrow(ctr_data_bench)), digits = 2)

# 4.2.2.2 - We bucketed durations 160+ months together because our test data contains later durations 
# that are not in our training data.  If we didn't do this then we wouldn't have predictions 
# for the later durations in the test data, which would cause the code to break when we score 
# the test data at the end.  Later we can try incorporating additional variables and 
# claim duration bucketing.
ctr_data_bench <- ctr_data_bench %>%
  mutate(ClaimDuration_v2_ = ifelse(ClaimDuration > 160, 160, ClaimDuration))

# select the variables needed to train and test the model and then aggregate the data 
# to speed up training.
agg_data <- ctr_data_bench %>%
  group_by(fold,
           Gender,
           ClaimDuration_v2_,
           ClaimType,
           LnCY_Yr_st,
           CY_Yr,
           ClaimDuration,
           Sample
           ) %>%
  summarise(exp_terms = sum(exp_terms),
            exp_terms_orig = sum(exp_terms_orig),
            Terminations = sum(Terminations),
            Exposure = sum(Exposure)
            ) %>% data.frame()

str(agg_data)

rm(ctr_data_bench)
gc()

# 4.2.3 - Now we have our data ready let's get ready for the regression function in 
# the glmnet package. It's a bit different than what we've seen so far. 
# glmnet() uses x, y, and offset input data instead of formulas like in glm().
# As a result, we will have to create a few different data sets.

# x: Select our independent variables 
ind_vars <- agg_data %>% 
  filter(Sample == "training") %>%
  select(Gender,
         ClaimType,
         ClaimDuration_v2_,
         LnCY_Yr_st
         ) 

# y: Grab our terminations
terminations <- agg_data %>% 
  filter(Sample == "training") %>%
  select(Terminations)

# offset: create expected termination offset
offset_cal <- agg_data %>% 
  filter(Sample == "training") %>%
  select(exp_terms) %>% 
  log() %>% # take the log since we are using a log-link function in the GLM
  as.matrix() 

# Next we need to prep our independent variables more for modeling.  For all categorical 
# variables we will build indicators for every level.  This is different than what is 
# typically done with a non-penalized glm where one would code one of the levels as a 
# reference level. This is known as one-hot encoding (building a full suite of dummy 
# variables). Doing so then allows the penalization to determine which variables should 
# be squeezed into the reference level. 
# You can choose a reference level yourself if you want, but this is not necessary with 
# a penalized regression. It can actually do the math when there is perfect 
# multicollinearity, unlike an OLS regression. This method of one-hot encoding is also 
# applicable to other machine learning methods such as tree based models. We will also 
# store the data in a sparse matrix via sparse.model.matrix() from the Matrix package, 
# which is an effective way to store data. It works by only storing the non-zero elements 
# of the data, saving a lot of space. Although not every package works with the 
# sparse matrix format, glmnet is a package that does work with this type of data.

# First we need to format all the categorical variables as factors to help the 
# sparse.model.matrix() function determine which variables to one-hot encode.
# Do this by listing out the variables we want to be factors
factor_vars <- c('Gender',
                 'ClaimType',
                 'ClaimDuration_v2_')

# Now update the variables in our list to be factors
ind_vars[, factor_vars] <- lapply(ind_vars[, factor_vars], as.factor)

# 4.2.4 - We are now ready to create our model. We will store our glm formula 
# in the glm_formula variable.
glm_formula <- formula(~ -1 # drop intercept since glmnet has a statment for this 
                       + Gender
                       + ClaimType
                       + LnCY_Yr_st
                       + Gender:ClaimType
                       + Gender:LnCY_Yr_st
                       + ClaimType:LnCY_Yr_st
                       + ClaimDuration_v2_:Gender
                       + ClaimDuration_v2_:ClaimType
                       )

# Here in one step we are one-hot encoding our factor variables we have stored in 
# the factor_vars list, building out our model design matrix we have specified in 
# the glm_formula from above and finally taking that design matrix and storing it 
# as a sparse formatted matrix that only points to the rows and
# columns of the non-zero elements in our design matrix
x_train_sp <- sparse.model.matrix(glm_formula,
                                  data = ind_vars,
                                  # This contrast call allows us to create indicators 
                                  # for every level of variables that have a factor format
                                  contrasts.arg = lapply(ind_vars[, sapply(ind_vars, is.factor)], 
                                                         contrasts, 
                                                         contrasts=FALSE)
                                  ) 
str(x_train_sp)
head(x_train_sp)

# store y as sparse matrix
term_sp <- terminations %>% 
  as.matrix() %>% 
  Matrix(., sparse=TRUE)

gc()

# Grab fold IDs and store them as a vector since glmnet needs them in this format for 
# the k-fold CV
fold_ID <- agg_data %>%
  filter(Sample == "training") %>%
  select(., fold) %>% 
  t() %>% as.vector()

# 4.2.5 - Check how our coefficients change as we vary the lambda penalty

# Now that we have added in some more variables and interactions that we are not sure
# should have an adjustment made for them we will use an alpha between 0 and 1.
# This is know as an elastic net regression, which provides feature selection 
# automatically by shrinking non-significant variables to zero.  When alpha is close to 
# 0 it set few variables to zero and as you move alpha closer to 1 it sets
# more variables to zero. Since the lasso penalty (alpha=1) tends to have trouble
# when faced with multicollinearity it is always good to use an alpha value of less
# than 1.0 to help guard against multicollinearity by blending in the ridge penalty. 
# The downfall with lasso is when there is multicollinearity within a group of
# variables lasso tends to randomly select one of the variables in the group
# and set the rest to zero. This can produce wild coefficients in your model.

# Typically the alpha value isn't too sensitive when varied between 0 and 1 so let's
# use a value of 0.5, which gives us a 50/50 blend of ridge and lasso
my_alpha = 0.5

glmnet_model1 <- glmnet(x = x_train_sp,
                        y = term_sp,
                        family = "poisson",
                        offset = offset_cal,
                        alpha = my_alpha,
                        standardize = FALSE, # set to false since binary variables shouldn't be standardized.  Only continuous ones should be
                        intercept = FALSE # set intercept to false since the intercept term is not penalized in the model and 
                                          # we only want to adjust the assumption if there is credible enough data to do so
                        )

# plot coefficient path
plot.glmnet(glmnet_model1, xvar = "lambda")
title(main = "Coefficients Across the Penalty Path\n\n")

# 4.2.6 - But wait... Most of the time the default lambda sequence doesn't show the 
# full lambda path

# Grab the lambda sequence that was used above and store to use with the k-fold CV.
# We'll stretch the lambda sequence a bit since usually the auto generated sequence 
# doesn't cover the full range of penalties (no penalty to full penalty).
my_lambda <- exp(
  seq(from=log(glmnet_model1$lambda[1]),
      to=log(glmnet_model1$lambda[1]*1e-8),
      length.out=100
  )
)

gc()

# Now let's view how our coefficients change with the full lambda sequence
glmnet_model2 <- glmnet(x = x_train_sp,
                        y = term_sp,
                        family = "poisson",
                        offset = offset_cal,
                        alpha = my_alpha,
                        lambda = my_lambda,
                        standardize = FALSE,
                        intercept = FALSE 
)

# plot coefficient path across the lambda penalties
# Note that this function plots the coefficients in log scale since
# they were fit using a log-link function
plot.glmnet(glmnet_model2, xvar = "lambda", ylab = "Coefficients (Log Scale)")
title(main = "Coefficients Across the Penalty Path\n\n")

# Though we can do this hack to convert them into factor adjustments
factor_coef_data <- glmnet_model2
factor_coef_data$beta <- exp(factor_coef_data$beta)
plot.glmnet(factor_coef_data, xvar = "lambda", ylab = "Coefficients (Factor Scale)")
title(main = "Coefficients Across the Penalty Path\n\n")

# Save the coefficient path in a CSV for easier viewing
all_coeff <- coef(glmnet_model2, s=my_lambda) %>% 
  exp() %>% as.matrix() %>% data.frame() 

coeff_seq <- data.frame(all_coeff)

# Save as csv file 
write.csv(coeff_seq, file = paste0(data_output, "\\", "0420_Full_coefficient_path.csv"))

# 4.2.7.1 - Now let's do a 10-fold cross-validation to select the optimal lambda  

# Create new instances of R to use for parallel processing the cross-validation
# via the registerDoParallel() function in the doParallel Package 
# Note, do not use more processes than there are folds
# Also do not use more processes than number of cores you have on your computer; 
# otherwise, this causes a traffic jam and slows things down
registerDoParallel(3)

glmnet_model2_cv <- cv.glmnet(x = x_train_sp,
                              y = term_sp,
                              family = "poisson",
                              offset = offset_cal,
                              alpha = my_alpha,
                              foldid = fold_ID,
                              lambda = my_lambda,
                              parallel = TRUE,
                              standardize = FALSE,
                              intercept = FALSE
                              )

# plot cross-validation results
plot(glmnet_model2_cv)
title(main = "K-fold Cross-validation Performance Across\nthe Penalty Path\n\n")


# 4.2.7.2 - plot coefficient path across the lambda penalties and add
# a red line that shows the optimal penalty chosen by the 10-fold CV
plot.glmnet(factor_coef_data, xvar = "lambda", ylab = "Coefficients (Factor Scale)")
abline(v = log(glmnet_model2_cv$lambda.min), col = "red")
title(main = "Coefficients Across the Penalty Path\n\n")

# Zoom in closer to the optimal penalty to get a better view of the coefficients 
plot.glmnet(factor_coef_data, xlim = c(-8, log(max(glmnet_model2_cv$lambda))),
            ylim = c(0.3, 1.5),
            xvar = "lambda", ylab = "Coefficients (Factor Scale)")
abline(v = log(glmnet_model2_cv$lambda.min), col = "red")
title(main = "Coefficients Across the Penalty Path\n\n")
gc()


# 4.2.8 - Now let's extract our Coefficients and predictions

# This one corresponds to the lambda that produces the min cross-validation error
Coeff_lambda_min <- coef(glmnet_model2_cv, s="lambda.min") %>% 
  as.matrix() %>% data.frame()

# This one corresponds to the lambda that is one standard error from the min cross-validation error
Coeff_lambda_1SE <- coef(glmnet_model2_cv, s="lambda.1se") %>% 
  as.matrix() %>% data.frame()

compare <- data.frame(Coeff_lambda_min[,-1],
                      lambda_min = exp(Coeff_lambda_min$X1),
                      lambda_1se = exp(Coeff_lambda_1SE$X1))


# We can save the coefficients as a csv file so we can view it outside of R.
# Looking at the file we can see that it made similar scalar adjustments as the raw AtoE
# factors we looked at in the beginning of this program. When models become more complex with
# additional variables and interactions, reviewing the relationships of the coefficients 
# becomes challenging. One approach for reviewing the final model for reasonableness 
# is to develop pseudo observations of every combination in the model and export predictions 
# for each combination.  Then from these predicted values you can create base tables of the 
# assumptions to review the relationships across different cells.  Another approach is to
# review the predictions on the historical experience for reasonableness, which we will
# do later in this program.
write.csv(compare, file = paste0(data_output, "\\", "0420_Coeff_lambda_min_and_1se.csv"))
gc()


# 4.2.9.1 - Now let's calculate the fit on our various data sets across the full 
# lambda path. Typically we would not do such a test on all data sets since testing 
# the various penalties on the data sets would bias our model.  And most of the time 
# you may not have enough data to afford to have data held out for a validation/test set.
# We are providing this illustration to help show how the k-fold cross-validation is a 
# good proxy of how your model will perform on an out-of-sample data set that was not 
# used to train the model.  It also illustrates how the choice of the selecting the 
# lambda.min and lambda.1se can involve some judgment, but usually both get you in the 
# same ballpark.
devi = function(type) {
  # Select our independent variables
  agg_vars <- agg_data %>% 
    filter(Sample == type) 
  
  # Now update the variables in our list to be factors
  agg_vars[, factor_vars] <- lapply(agg_vars[, factor_vars], as.factor)
  
  x_vars <- sparse.model.matrix(glm_formula,
                                data = agg_vars,
                                contrasts.arg = lapply(agg_vars[, sapply(agg_vars, 
                                                                         is.factor)], 
                                                       contrasts, contrasts=FALSE)
                                ) 
  
  # Grab our terminations
  y <- agg_data %>% 
    filter(Sample == type) %>%
    select(Terminations)
  
  # create exposure offset
  offset <- agg_data %>% 
    filter(Sample == type) %>%
    select(exp_terms) %>% 
    log() %>% # take the log since we are using a log-link function in the GLM
    as.matrix()
  
  # predict at every lambda level
  eta <- predict(glmnet_model2, newx=x_vars, newoffset=offset, type='link')
  
  # Poisson deviance formula taken from glmnet code
  # https://github.com/cran/glmnet/blob/master/R/cv.fishnet.R  
  deveta = y$Terminations * eta - exp(eta)
  devy = y$Terminations * log(y$Terminations) - y$Terminations
  devy[y$Terminations == 0] = 0
  
  apply(2*(devy - deveta), 2, mean, na.rm = TRUE)
}

agg_data %>% distinct(Sample)


deviance_cal = devi("training")
deviance_val = devi("validation")
deviance_test = devi("testing")

min_lambdas <- data.frame(train.min = which.min(deviance_cal),
                          cv.min = which.min(glmnet_model2_cv$cvm),
                          val.min = which.min(deviance_val),
                          test.min = which.min(deviance_test))

row.names(min_lambdas) <- "Lowest Error Lambda"

# 4.2.9.2 - First look at the training and 10-fold CV error
# we will drop the values on the y-axis since we only care 
# about the location of the lowest error on these curves when 
# trying to select the lambda values
plot(glmnet_model2_cv, 
     ylab='Error', 
     ylim = c(min(deviance_cal), max(glmnet_model2_cv$cvm)), 
     yaxt='n')
title(main = "Performance Across the Penalty Path\nfor Various Data Sets\n\n")

# the "best" lambda corresponds to the lowest point on this curve, which is now shown by the
# vertical red line
abline(v = log(glmnet_model2$lambda[min_lambdas$cv.min]), col = "red")

# 4.2.9.3 - Add on the training error curve
points(log(glmnet_model2$lambda), deviance_cal, col="blue", pch="*")  
legend("topleft", c("10 fold CV", "Train"), pch="*", 
       col=c("red", "blue"), lty=1:3, cex=0.8) 

# As you can see with the blue vertical line the best fit based on the
# training error suggests using no penalty, which will always be the 
# case when looking at the training error. 
abline(v = log(glmnet_model2$lambda[min_lambdas$train.min]), col = "blue")


# 4.2.9.4 - Now let's see where the lowest error is on the validation set 
# as you can see with the green vertical line the 10-fold CV did a pretty
# good job at selecting a lambda that would perform well on the validation set
legend("topleft", c("10 fold CV", "Train", "Validation"), pch="*", 
       col=c("red", "blue", "green"), lty=1:3, cex=0.8) 
abline(v = log(glmnet_model2$lambda[min_lambdas$val.min]), col = "green")


# 4.2.9.5 - and finally the test set
# again we can see that the 10-fold CV did a good job at selecting an optimal penalty
# such that our out-of-time sample test set had the best performance close to where
# the 10-fold CV selected the optimal penalty
legend("topleft", c("10 fold CV", "Train", "Validation", "Test"), pch="*", 
       col=c("red", "blue", "green", "orange"), lty=1:3, cex=0.8)
abline(v = log(glmnet_model2$lambda[min_lambdas$test.min]), col = "orange")

# Looking at the lambda index chosen for each we can see that the
# 10-fold CV did a good job at picking a lambda value that worked well on the
# validation and test sets. If we based our decision on selecting the penalty 
# that produced the lowest calibration error then we would have no penalty 
# at all and would have overfit the data.  This highlights the ability 
# of the penalized GLMs to automate the bias-variance trade-off.
min_lambdas

gc()


# 4.2.10.1 - After reviewing our results now let's test the fit further

# First we'll re-score all the data 
# create exposure offset
offset <- agg_data %>%
  mutate(LnExp_Term = log(exp_terms)) %>%
  select(LnExp_Term) %>% 
  as.matrix() 

# Convert our variables in our factor_vars list to factors
agg_data[, factor_vars] <- lapply(agg_data[, factor_vars], as.factor)
gc()

# create our sparse matrix of indicator variables
x_all <- sparse.model.matrix(glm_formula,
                             data = agg_data,
                             contrasts.arg = lapply(agg_data[, sapply(agg_data, is.factor)], 
                                                    contrasts,
                                                    contrasts=FALSE)
                             ) 
gc()

# Make some predictions at lambda.1se, lambda.min and no penalty
lambda_1se <- predict(glmnet_model2_cv,
                      newx = x_all,
                      newoffset = offset,
                      type = 'response',
                      s = "lambda.1se")

lambda_min <- predict(glmnet_model2_cv,
                      newx = x_all,
                      newoffset = offset,
                      type = 'response',
                      s = "lambda.min")

no_penalty <- predict(glmnet_model2_cv,
                      newx = x_all,
                      newoffset = offset,
                      type = 'response',
                      s = 0)
gc()

predictions <- data.frame(agg_data,
                          lambda_1se = lambda_1se[,1],
                          lambda_min = lambda_min[,1],
                          no_penalty = no_penalty[,1])
rm(lambda_min,
   no_penalty,
   lambda_1se)

# 4.2.10.2 - First view overall AtoEs on our three training sets.
AtoE_sample <- predictions %>%
  group_by(Sample) %>%
  summarise(Termination = comma(sum(Terminations)),
            Rate = percent(sum(Terminations)/sum(Exposure)),
            AtoE_bench = round(sum(Terminations)/sum(exp_terms_orig), 2),
            AtoE_bench_adj = round(sum(Terminations)/sum(exp_terms), 2),
            AtoE_lambda_1se = round(sum(Terminations)/sum(lambda_1se), 2),
            AtoE_lambda_min = round(sum(Terminations)/sum(lambda_min), 2),
            AtoE_no_penalty = round(sum(Terminations)/sum(no_penalty), 2))
View(AtoE_sample)

# create function to test AtoEs on various variables by sample
AtoE <- function(test){
  AtoE_sample <- predictions %>%
    group_by_('Sample',
              test) %>%
    summarise(Termination = comma(sum(Terminations)),
              Rate = percent(sum(Terminations)/sum(Exposure)),
              AtoE_bench = round(sum(Terminations)/sum(exp_terms_orig), 2),
              AtoE_bench_adj = round(sum(Terminations)/sum(exp_terms), 2),
              AtoE_lambda_1se = round(sum(Terminations)/sum(lambda_1se), 2),
              AtoE_lambda_min = round(sum(Terminations)/sum(lambda_min), 2),
              AtoE_no_penalty = round(sum(Terminations)/sum(no_penalty), 2))
  View(AtoE_sample)
  
}
AtoE('Gender')
AtoE('ClaimType')
AtoE('CY_Yr')

AtoE_sample <- predictions %>%
  group_by(Sample,
           ClaimType,
           CY_Yr) %>%
  summarise(Termination = comma(sum(Terminations)),
            Rate = percent(sum(Terminations)/sum(Exposure)),
            AtoE_bench = round(sum(Terminations)/sum(exp_terms_orig), 2),
            AtoE_bench_adj = round(sum(Terminations)/sum(exp_terms), 2),
            AtoE_lambda_1se = round(sum(Terminations)/sum(lambda_1se), 2),
            AtoE_lambda_min = round(sum(Terminations)/sum(lambda_min), 2),
            AtoE_no_penalty = round(sum(Terminations)/sum(no_penalty), 2))
View(AtoE_sample)

# 4.2.11 - Check out mean squared error (MSE)
# the no penalty performs better on the training data because MSE is lower,
# but lambda.min performs better on both the validation and testing
MSE_sample <- predictions %>%
  group_by(Sample) %>%
  summarise(Termination = comma(sum(Terminations)),
            Rate = percent(sum(Terminations)/sum(Exposure)),
            mse_bench = round(sum((Terminations-exp_terms_orig)^2)/sum(Exposure), 5),
            mse_bench_adj = round(sum((Terminations-exp_terms)^2)/sum(Exposure), 5),
            mse_lambda_1se = round(sum((Terminations-lambda_1se)^2)/sum(Exposure), 5),
            mse_lambda_min = round(sum((Terminations-lambda_min)^2)/sum(Exposure), 5),
            mse_no_penalty = round(sum((Terminations-no_penalty)^2)/sum(Exposure), 5))
View(MSE_sample)

# create function to test MSE on various variables by sample
MSE <- function(test){
  MSE_sample <- predictions %>%
    group_by_('Sample',
              test) %>%
    summarise(Termination = comma(sum(Terminations)),
              Rate = percent(sum(Terminations)/sum(Exposure)),
              mse_bench = round(sum((Terminations-exp_terms_orig)^2)/sum(Exposure), 5),
              mse_bench_adj = round(sum((Terminations-exp_terms)^2)/sum(Exposure), 5),
              mse_lambda_1se = round(sum((Terminations-lambda_1se)^2)/sum(Exposure), 5),
              mse_lambda_min = round(sum((Terminations-lambda_min)^2)/sum(Exposure), 5),
              mse_no_penalty = round(sum((Terminations-no_penalty)^2)/sum(Exposure), 5))
  View(MSE_sample)
}
MSE('Gender')
MSE('ClaimType')
MSE('CY_Yr')


# 4.2.12.1 - Now let's view some graphs by duration and Sample
plot_test <- function(val){
  agg_results <- predictions %>%
    filter(Sample == val) %>%
    group_by(ClaimDuration) %>%
    summarise(actual_hz_rate = sum(Terminations)/sum(Exposure),
              bench = sum(exp_terms_orig)/sum(Exposure),
              bench_adj = sum(exp_terms)/sum(Exposure),
              no_penalty = sum(no_penalty)/sum(Exposure),
              lambda_min = sum(lambda_min)/sum(Exposure),
              lambda_1se = sum(lambda_1se)/sum(Exposure)
              )
  
  ggplotly(ggplot() + 
             geom_line(data = agg_results, 
                       linetype = "dotted", 
                       aes(x = ClaimDuration, y = actual_hz_rate, col="actual")
                       )+ 
             geom_line(data = agg_results, 
                       linetype = "dashed", 
                       aes(x = ClaimDuration, y = bench, col="bench")
                       )+
             
             geom_line(data = agg_results, 
                       linetype = "dashed", 
                       aes(x = ClaimDuration, y = bench_adj, col="bench_adj")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = no_penalty, col="no_penalty")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = lambda_min, col="lambda_min")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = lambda_1se, col="lambda_1se")
                       )+
             ggtitle(paste0("Actual vs Predicted Hazard Rates on ", val," Holdout"))
           )
}
plot_test('training')
plot_test('validation')
plot_test('testing')

# 4.2.12.2 - View results on testing set for Gender
plot_test <- function(val, set){
  agg_results <- predictions %>%
    filter(Gender == val,
           Sample == set) %>%
    group_by(ClaimDuration) %>%
    summarise(actual_hz_rate = sum(Terminations)/sum(Exposure),
              bench = sum(exp_terms_orig)/sum(Exposure),
              bench_adj = sum(exp_terms)/sum(Exposure),
              no_penalty = sum(no_penalty)/sum(Exposure),
              lambda_min = sum(lambda_min)/sum(Exposure),
              lambda_1se = sum(lambda_1se)/sum(Exposure))
  
  ggplotly(ggplot() + 
             geom_line(data = agg_results, 
                       linetype = "dotted", 
                       aes(x = ClaimDuration, y = actual_hz_rate, col="actual")
                       )+ 
             geom_line(data = agg_results, 
                       linetype = "dashed", 
                       aes(x = ClaimDuration, y = bench, col="bench")
                       )+
             geom_line(data = agg_results, 
                       linetype = "dashed", 
                       aes(x = ClaimDuration, y = bench_adj, col="bench_adj")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = no_penalty, col="no_penalty")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = lambda_min, col="lambda_min")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = lambda_1se, col="lambda_1se")
                       )+
             ggtitle(paste0("Actual vs Predicted Hazard Rates for ", 
                            val, "\non the ", set, " Set"))
           )
}
plot_test('Female', "testing")
plot_test('Male', "testing")


# 4.2.12.3 - View results on testing set for ClaimType
plot_test <- function(val, set){
  agg_results <- predictions %>%
    filter(ClaimType == val,
           Sample == set) %>%
    group_by(ClaimDuration) %>%
    summarise(actual_hz_rate = sum(Terminations)/sum(Exposure),
              bench = sum(exp_terms_orig)/sum(Exposure),
              bench_adj = sum(exp_terms)/sum(Exposure),
              no_penalty = sum(no_penalty)/sum(Exposure),
              lambda_min = sum(lambda_min)/sum(Exposure),
              lambda_1se = sum(lambda_1se)/sum(Exposure))
  
  ggplotly(ggplot() + 
             geom_line(data = agg_results, 
                       linetype = "dotted", 
                       aes(x = ClaimDuration, y = actual_hz_rate, col="actual")
                       )+ 
             geom_line(data = agg_results, 
                       linetype = "dashed", 
                       aes(x = ClaimDuration, y = bench, col="bench")
                       )+
             geom_line(data = agg_results, 
                       linetype = "dashed", 
                       aes(x = ClaimDuration, y = bench_adj, col="bench_adj")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = no_penalty, col="no_penalty")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = lambda_min, col="lambda_min")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = lambda_1se, col="lambda_1se")
                       )+
             ggtitle(paste0("Actual vs Predicted Hazard Rates for ", 
                            val, "\non the ", set, " Set"))
           )
}
plot_test('HHC', "testing")
plot_test('NH', "testing")
plot_test('ALF', "testing")


# 4.2.12.4 - View results on testing set for CY_Yr
plot_test <- function(val, set){
  agg_results <- predictions %>%
    filter(CY_Yr == val,
           Sample == set) %>%
    group_by(ClaimDuration) %>%
    summarise(actual_hz_rate = sum(Terminations)/sum(Exposure),
              bench = sum(exp_terms_orig)/sum(Exposure),
              bench_adj = sum(exp_terms)/sum(Exposure),
              no_penalty = sum(no_penalty)/sum(Exposure),
              lambda_min = sum(lambda_min)/sum(Exposure),
              lambda_1se = sum(lambda_1se)/sum(Exposure))
  
  ggplotly(ggplot() + 
             geom_line(data = agg_results, 
                       linetype = "dotted", 
                       aes(x = ClaimDuration, y = actual_hz_rate, col="actual")
                       )+ 
             geom_line(data = agg_results, 
                       linetype = "dashed", 
                       aes(x = ClaimDuration, y = bench, col="bench")
                       )+
             geom_line(data = agg_results, 
                       linetype = "dashed", 
                       aes(x = ClaimDuration, y = bench_adj, col="bench_adj")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = no_penalty, col="no_penalty")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = lambda_min, col="lambda_min")
                       )+
             geom_line(data = agg_results, 
                       aes(x = ClaimDuration, y = lambda_1se, col="lambda_1se")
                       )+
             ggtitle(paste0("Actual vs Predicted Hazard Rates for ", 
                            val, "\non Testing Holdout"))
           )
}
plot_test(2010, "testing")
plot_test(2011, "testing")

# 4.2.13 - After you have selected a strategy for developing a model on the 
# training data and have confirmed the approach holds up on the validation and 
# testing set and have also reviewed the model for reasonableness, retrain 
# the model on all the data to incorporate the more recent data.  
# You should then review the final model that was trained on all the data again 
# for reasonableness. At this point you may also want to make manual smoothing 
# adjustments to the final assumption produced by the model if necessary.